/*
 * Camera.h
 *
 *  Created on: 07-03-2013
 *      Author: crm
 */

#pragma once

#include "asmmath.h"

#include "../orientation.h"

namespace Engine
	{
	class Camera: public Orientation
		{
		public:
			enum Type
				{
				ORTHOGRAPHIC,
				PERSPECTIVE,
				GUI,
				};

		protected:

			Type type;

			float width, height;
			float fov;
			float cutnear, cutfar;

		public:
			Camera():
				Orientation(AVector(0, 0, 0), AVector(0, 0, -1), AVector(0, 1, 0)),
				type(ORTHOGRAPHIC), fov(60.0f), cutnear(1.0f), cutfar(10000.0f) {}
			Camera(Type type, float width, float height, float cutnear, float cutfar, float fov=60.0f):
				Orientation(AVector(0, 0, 0), AVector(0, 0, -1), AVector(0, 1, 0)),
				type(type), width(width), height(height), fov(fov), cutnear(cutnear), cutfar(cutfar)
				{
				//
				}
			virtual ~Camera() {}

			void update();

			inline float getWidth() const {return width;}
			inline float getHeight() const {return height;}
			inline AVector getSize() const {return AVector(width, height);}
			inline Type getType() const {return type;}
			inline float getFov() const {return fov;}

			AMatrix getProjection() const;
			AMatrix getProjection(int width, int height) const;
			AMatrix getView() const;

			void setType(Type s) {type=s;}
			void setFov(float s) {if(fov>0.0f && fov<180.0f) fov=s;}
			void setZoom(float s) {if(s>0.0f) setScale(AVector(s, s, s));}

			Orientation getBillboard(const AVector& objpos) const;
			AVector getPointOnScreen(const AVector& objpos) const;
			void getRay(int mx, int my, AVector& raypos, AVector& raydir) const;	// Pozycja myszy, [cx, cw], [xy, ch]
		};
	}
