/*
 * framebuffer.cpp
 *
 *  Created on: 28 mar 2016
 *      Author: crm
 */

#include "framebuffer.h"

#include <gl/glu.h>

#include "log.h"

using namespace Engine;

bool FrameBuffer::init(int w, int h)
	{
	clear();

	Log::debug("FrameBuffer.init: Tworzenie FrameBuffera, %dx%dpx", w, h);

	GLuint err;

	// Renderbuffer
	GLint boundrbid;
	glGetIntegerv(GL_TEXTURE_BINDING_2D, &boundrbid);

	glGenRenderbuffers(1, &rbid);

	if(!rbid)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie utworzyc RenderBuffer");
		return false;
		}

	glBindRenderbuffer(GL_RENDERBUFFER, rbid);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, w, h); // GL_DEPTH_COMPONENT powoduje automatyczny wybor precyzji depth buffera

	glBindRenderbuffer(GL_RENDERBUFFER, boundrbid);

	if((err=glGetError())!=GL_NO_ERROR)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie zajac pamieci na RenderBuffer");
		Log::fail("FrameBuffer.init: %s", gluErrorString(err));
		return false;
		}
	else
		Log::debug("FrameBuffer.init: RenderBuffer zrobiony");

	// Tekstura
	GLint boundtid;
	glGetIntegerv(GL_TEXTURE_BINDING_2D, &boundtid);

	glGenTextures(1, &tid);

	if(!tid)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie utworzyc tekstury");
		return false;
		}

	glBindTexture(GL_TEXTURE_2D, tid);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);

	glBindTexture(GL_TEXTURE_2D, boundtid);

	if((err=glGetError())!=GL_NO_ERROR)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie zajac pamieci na teksture");
		Log::fail("FrameBuffer.init: %s", gluErrorString(err));
		return false;
		}
	else
		Log::debug("FrameBuffer.init: Tekstura zrobiona");

	// Framebuffer
	GLint boundfbid;
	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &boundfbid);

	glGenFramebuffers(1, &fbid);

	if(!fbid)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie utworzyc FrameBuffer");
		return false;
		}

	glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fbid);

	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, tid, 0);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, rbid);

	if(glCheckFramebufferStatus(GL_FRAMEBUFFER)!=GL_FRAMEBUFFER_COMPLETE)
		{
		Log::fail("FrameBuffer.init: FrameBuffer nie jest kompletny");

		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, boundfbid);

		return false;
		}
	else
		Log::ok("FrameBuffer.init: Utworzono FrameBuffer [GLid: %u]", fbid);

	glClearColor(1, 1, 1, 1);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glBindFramebuffer(GL_DRAW_FRAMEBUFFER, boundfbid);

	if((err=glGetError())!=GL_NO_ERROR)
		{
		Log::fail("FrameBuffer.init: Nie udalo sie zbindowac poprzedniego FBO (id: %u)", boundfbid);
		Log::fail("FrameBuffer.init: %s", gluErrorString(err));
		return false;
		}

	this->w=w;
	this->h=h;

	//cam=Camera(Camera::ORTHOGRAPHIC, w, h, 1, 10000);

	return true;
	}

void FrameBuffer::clear()
	{
	if(!fbid)
		return;

	if(rbid)
		glDeleteRenderbuffers(1, &rbid);

	if(tid)
		glDeleteTextures(1, &tid);

	if(fbid)
		glDeleteFramebuffers(1, &fbid);

	Log::debug("FrameBuffer.clear: Kasowanie FBO [GLid: %u]", fbid);

	rbid=0u;
	tid=0u;
	fbid=0u;
	}

