/*
 * render.h
 *
 *  Created on: 9 mar 2016
 *      Author: crm
 */

#pragma once

#include <string>

#include "camera.h"
#include "framebuffer.h"
#include "vertexbuffer.h"
#include "shader.h"
#include "image.h"
#include "imageptr.h"
#include "sprite.h"
#include "spriteptr.h"

#include "asmmath.h"


namespace Engine
	{
	namespace Render
		{
		enum Mode
			{
			MODE_DEFAULT,
			MODE_GUI,
			MODE_GUI_ITEM,
			};

		/*enum CameraID
			{
			CAMERA_DEFAULT,
			CAMERA_GUI,

			CAMERA_COUNT,
			};*/

		bool init(int w, int h, const std::string& title, const std::string& icon="", bool fullscreen=false);
		void update();
		void clear();

		bool setResolution(int w, int h, bool fullscreen);
		bool setFullscreen(bool fullscreen);
		bool toggleFullscreen();

		AVector getScreenSize();

		void pushState();
		void popState();

		void bind(GLuint tid);
		void bind(const ImagePtr& img);
		void bind(const SpritePtr& spr);

		void setShader(Shader& shader);
		void setEffect(Shader& shader);
		void unsetEffect();
		//void applyEffect();
		void setFrameBuffer(FrameBuffer& fbo, bool clear=true);
		void setFrameBuffer();
		//void setFrameBufferGUI();

		void setMode(Mode id);
		void setCamera(Camera *cam);
		void setColor(const AVector& color);
		void setAlpha(float alpha);

		void drawLine(const AVector& a, const AVector& b);
		void drawTriangle(const AVector& a, const AVector& b, const AVector& c);
		void drawQuad(const AVector& a, const AVector& b, const AVector& c, const AVector& d);
		void drawRectLine(const AVector& a, const AVector& b);
		void drawTexturedTriangle(const AVector& a, const AVector& b, const AVector& c, const AVector& ta, const AVector& tb, const AVector& tc);
		void drawTexturedRect(const AVector& a, const AVector& b, const AVector& c, const AVector& d, const AVector& ta, const AVector& tb, const AVector& tc, const AVector& td);

		void print(const FrameBuffer& fbo, const Orientation& orientation, float x, float y, float w, float h);
		void print(const Image& img, const Orientation& orientation, float x, float y, float w, float h);
		void print(const ImagePtr& iptr, const Orientation& orientation, float x, float y, float w, float h);
		void print(const Sprite& spr, const Orientation& orientation, unsigned animation, unsigned direction, unsigned frame);
		void print(const SpritePtr& sptr, const Orientation& orientation, bool rotate=false);

		void skybox(const Image& img, float x, float y, float w, float h);
		void skybox(const ImagePtr& iptr, float x, float y, float w, float h);
		void skybox(const Sprite& spr, unsigned animation, unsigned direction, unsigned frame);
		void skybox(const SpritePtr& sptr);
		}
	}
