/*
 * shader.h
 *
 *  Created on: 11 mar 2016
 *      Author: crm
 */

#pragma once

#include <map>
#include <string>
#include <gl/glee.h>
#include <gl/gl.h>

#include "asmmath.h"

namespace Engine
	{
	class Shader
		{
		private:
			struct Uniform
				{
				enum class Type
					{
					INT,
					FLOAT,
					};

				Uniform(): type(Type::INT) {value.uint=0;}
				Uniform(int v): type(Type::INT) {value.uint=v;}
				Uniform(float v): type(Type::FLOAT) {value.ufloat=v;}
				Uniform(const Uniform& u): type(u.type), value(u.value) {}

				Type type;

				union
					{
					int uint;
					float ufloat;
					} value;
				};

			Shader(const Shader& i): program(i.program) {}
			Shader& operator=(const Shader& i) {program=i.program;return *this;}

			GLuint program;
			std::map<std::string, GLuint> uniforms;
			std::map<std::string, Uniform> uvalues;

		public:
			Shader(): program(0u) {}
			virtual ~Shader() {}

			bool load(const std::string& path);
			void clear();

			GLuint getUniform(const std::string& name);
			GLuint getProgramID() const {return program;}

			void uniform(const std::string& name, int v) {uvalues[name]=Uniform(v);}
			void uniform(const std::string& name, float v) {uvalues[name]=Uniform(v);}

			void setUniforms();
		};
	}
