/*
 * sprite.h
 *
 *  Created on: 22-02-2013
 *      Author: crm
 */

#pragma once

#include <vector>
#include <string>
#include <map>

#include "../refcounted.h"
#include "imageptr.h"

namespace Engine
	{
	class Sprite: public RefCounted
		{
		public:
			enum class DirectionName: unsigned
				{
				RIGHT=0,
				RIGHT_DOWN=1,
				DOWN=2,
				LEFT_DOWN=3,
				LEFT=4,
				LEFT_UP=5,
				UP=6,
				RIGHT_UP=7,

				COUNT=8,

				DEFAULT=DOWN
				};

			struct Frame
				{
				Frame(int x, int y, int w, int h, int spotx=0, int spoty=0, int actx=0, int acty=0):
					x(x), y(y), w(w), h(h), spotx(spotx), spoty(spoty), actx(actx), acty(acty)
					{}

				int x, y, w, h, spotx, spoty, actx, acty;
				};

			class Direction
				{
				private:
					float spd;
					unsigned ret;

					std::vector<Frame> frames;

				public:
					 Direction(): spd(0.0f), ret(0u) {};
					~Direction() {clear();}

					void clear();

					inline float getSpeed() const {return spd;}
					inline unsigned getReturnFrame() const {return ret;}
					inline unsigned getFrameCount() const {return frames.size();}
					inline const Frame& getFrame(unsigned i) const {return frames[i];}

					inline void setSpeed(float s) {spd=s;}
					inline void setReturn(unsigned s) {ret=s;}

					inline void addFrame(const Frame& f) {frames.push_back(f);}
				};

			class Animation
				{
				private:
					Direction *directions[(int)DirectionName::COUNT];

				public:
					 Animation() {init();}
					~Animation() {clear();}

					void init();	// inicjalizacja domyslnego kierunku, wywolywane z konstruktora
					void clear();

					inline const Direction& getDirection(DirectionName dir) const {return *directions[(int)dir];}

					void addDirection(DirectionName dir, Direction *pdir);
				};

		private:
			ImagePtr image;

			std::vector<Animation *> animations;
			std::map<std::string, Animation *> named;

			std::string name;

		public:
			Sprite(): image(), name() {}
			virtual ~Sprite() {clear();}

			bool init(const std::string& name);
			void clear();

			inline ImagePtr getImage() const {return image;}
			inline const Animation& getAnimation(unsigned i) const {return *animations[i];}
			inline const Animation& getAnimation(const std::string& name) const {return *named.find(name)->second;}
			inline unsigned getAnimationCount() const {return animations.size();}

			inline std::string getName() const {return name;}

			unsigned getAnimationID(const Animation *a) const;
			unsigned getAnimationID(const std::string& name) const;

			void setImage(const ImagePtr& s) {image=s;}

			unsigned addAnimation(Animation *ani, const std::string& name="");
		};
	}
