/*
 * spriteptr.cpp
 *
 *  Created on: 23-02-2013
 *      Author: crm
 */

#include "sprite.h"
#include "spriteptr.h"
#include "spritemanager.h"

using namespace Engine;

SpritePtr::SpritePtr():
	sprite(NULL), animationID(0u), directionID((int)Sprite::DirectionName::DEFAULT), animation(0u), direction(0u), frame(0.0f), alpha(1.0f)
	{
	setSprite(NULL);
	}

SpritePtr::SpritePtr(const Sprite *s):
	sprite(NULL), animationID(0u), directionID((int)Sprite::DirectionName::DEFAULT), animation(0u), direction(0u), frame(0.0f), alpha(1.0f)
	{
	setSprite(s);
	}

SpritePtr::SpritePtr(const SpritePtr& p):
	sprite(NULL), animationID(0u), directionID((int)Sprite::DirectionName::DEFAULT), animation(0u), direction(0u), frame(0.0f), alpha(1.0f)
	{
	setSprite(p.sprite);
	}

SpritePtr::SpritePtr(const std::string& name):
	sprite(NULL), animationID(0u), directionID((int)Sprite::DirectionName::DEFAULT), animation(0u), direction(0u), frame(0.0f), alpha(1.0f)
	{
	setSprite(name);
	}

SpritePtr::~SpritePtr()
	{
	clear();
	}


void SpritePtr::clear()
	{
	setSprite(NULL);
	}

void SpritePtr::update(float dt)
	{
	frame+=direction->getSpeed()*dt;

	if(frame>=direction->getFrameCount())
		frame=direction->getReturnFrame()+(frame-(int)frame);
	}


bool SpritePtr::loop() const
	{
	return direction->getSpeed()==0.0f || direction->getReturnFrame()<direction->getFrameCount()-1u;
	}

bool SpritePtr::ended() const
	{
	return loop() || frame>=direction->getReturnFrame();
	}


void SpritePtr::setSprite(const Sprite *s)
	{
	if(sprite==s)
		return;

	if(sprite)
		{
		sprite->unref();

		if(!sprite->refs())
			SpriteManager::free(sprite, true);
		}

	sprite=s;

	if(sprite)
		{
		sprite->ref();
		animationID=-1;

		setAnimation(0);
		}
	}

void SpritePtr::setSprite(const SpritePtr& p)
	{
	setSprite(p.sprite);
	}

void SpritePtr::setSprite(const std::string& name)
	{
	setSprite(SpriteManager::get(name));
	}


void SpritePtr::setAnimation(unsigned sa)
	{
	if(animationID==sa)
		return;

	animation=&sprite->getAnimation(sa);
	animationID=sa;

	unsigned tmp=directionID;
	directionID=-1;
	setDirection(tmp);
	}

void SpritePtr::setAnimation(const std::string& sa)
	{
	if(animationID==sprite->getAnimationID(sa))
		return;

	animation=&sprite->getAnimation(sa);
	animationID=sprite->getAnimationID(animation);

	unsigned tmp=directionID;
	directionID=-1;
	setDirection(tmp);
	}

void SpritePtr::setDirection(unsigned sd)
	{
	if(directionID==sd)
		return;
	if(sd>=(int)Sprite::DirectionName::COUNT)
		sd=sd%((int)Sprite::DirectionName::COUNT);

	direction=&animation->getDirection((Sprite::DirectionName)sd);
	directionID=sd;

	frame=0;
	}


void SpritePtr::getSize(int& gw, int& gh) const
	{
	const Sprite::Frame& f=direction->getFrame(frame);

	gw=f.w;
	gh=f.h;
	}

int SpritePtr::getW() const
	{
	return direction->getFrame(frame).w;
	}

int SpritePtr::getH() const
	{
	return direction->getFrame(frame).h;
	}

void SpritePtr::getHotspot(int& gx, int& gy) const
	{
	const Sprite::Frame& f=direction->getFrame(frame);

	gx=f.spotx;
	gx=f.spoty;
	}

int SpritePtr::getHotspotX() const
	{
	return direction->getFrame(frame).spotx;
	}

int SpritePtr::getHotspotY() const
	{
	return direction->getFrame(frame).spoty;
	}

void SpritePtr::getActionPoint(int& gx, int& gy) const
	{
	const Sprite::Frame& f=direction->getFrame(frame);

	gx=f.actx;
	gy=f.acty;
	}

int SpritePtr::getActionPointX() const
	{
	return direction->getFrame(frame).actx;
	}

int SpritePtr::getActionPointY() const
	{
	return direction->getFrame(frame).acty;
	}

unsigned SpritePtr::getAnimationCount() const
	{
	return sprite->getAnimationCount();
	}

unsigned SpritePtr::getFrameCount() const
	{
	return direction->getFrameCount();
	}
