/*
 * spriteptr.h
 *
 *  Created on: 23-02-2013
 *      Author: crm
 */

#pragma once

#include <string>

#include "sprite.h"

namespace Engine
	{
	class SpritePtr
		{
		private:
			const Sprite *sprite;

			unsigned animationID;
			unsigned directionID;

			const Sprite::Animation *animation;
			const Sprite::Direction *direction;
			float frame;

			float alpha;

		public:
			 SpritePtr();
			 SpritePtr(const Sprite *s);
			 SpritePtr(const SpritePtr& p);
			 SpritePtr(const std::string& name);
			~SpritePtr();

			SpritePtr& operator=(const SpritePtr& s) {setSprite(s); return *this;}

			inline const Sprite& operator*() const	{return *sprite;}
			inline const Sprite* operator->() const	{return sprite;}
			inline operator bool() const			{return !!sprite;}
			inline operator const Sprite*() const	{return sprite;}

			void clear();
			void update(float dt);

			bool loop() const;
			bool ended() const;

			inline const Sprite* getSprite() const					 {return sprite;}
			inline const Sprite::Animation* getAnimation() const	 {return animation;}
			inline const Sprite::Direction* getDirection() const	 {return direction;}
			inline unsigned getAnimationID() const					 {return animationID;}
			inline unsigned getDirectionID() const					 {return directionID;}
			inline unsigned getFrame() const						 {return frame;}

			void getSize(int& gw, int& gh) const;
			int getW() const;
			int getH() const;
			void getHotspot(int& gx, int& gy) const;
			int getHotspotX() const;
			int getHotspotY() const;
			void getActionPoint(int& gx, int& gy) const;
			int getActionPointX() const;
			int getActionPointY() const;
			unsigned getAnimationCount() const;
			unsigned getFrameCount() const;
			float getAlpha() const {return alpha;}

			void setSprite(const Sprite *s);
			void setSprite(const SpritePtr& p);
			void setSprite(const std::string& name);
			void setAnimation(unsigned sa);
			void setAnimation(const std::string& sa);
			void setDirection(unsigned sd);
			void setAlpha(float s) {alpha=s;}
		};
	}
