/*
 * sound2.h
 *
 *  Created on: 23-07-2013
 *      Author: crm
 */

#pragma once

#include <string>
#include <vector>
#include <SDL/SDL_mixer.h>

namespace Engine
	{
	class Sound
		{
		private:
			struct Sample
				{
				 Sample(): snd(NULL), data(NULL), size(0u), name("") {}
				 Sample(const Sample& s): snd(s.snd), data(s.data), size(s.size), name(s.name) {}
				~Sample() {if(data) delete [] data; if(snd) Mix_FreeChunk(snd);}

				Mix_Chunk *snd;

				char *data;
				unsigned size;

				std::string name;
				};

			Sample* findSoundByName(const std::string& name);
			Sample* load(const std::string& name);

			std::vector<Sample *> samples;

			float volumesound;
			float volumemusic;

		public:
			Sound(): volumesound(1.0f), volumemusic(1.0f) {}

			bool init(int channels, int sampling);
			void update(float dt);
			void clear();

			int sound(const std::string& name, float volume=1.0f);
			int music(const std::string& name, float volume=1.0f);

			int play(const std::string& name, int channel, float volume, bool repeat, float pan, float pitch);
			void pause(int channel);
			void resume(int channel);
			void stop(int channel);
			void rewind(int channel);

			bool isPlaying(int channel);

			void setGlobalSoundVolume(float vol);	// vol jest znormalizowane (0.0f-1.0f)
			void setGlobalMusicVolume(float vol);	// vol jest znormalizowane (0.0f-1.0f)

			void setPitch(int channel, float val, int timeout=0);
			void setVolume(int channel, float val, int timeout=0);
			void setPan(int channel, float val, int timeout=0);

			float getGlobalSoundVolume();
			float getGlobalMusicVolume();

			float getPitch(int channel);
			float getVolume(int channel);
			float getPan(int channel);
		};
	}
