/*
 * ativeobject.cpp
 *
 *  Created on: 1 maj 2016
 *      Author: crm
 */

#include "activeobject.h"

#include <cmath>

#include "log.h"

#include "../engine/render/render.h"
#include "objecttemplate.h"
#include "level.h"

template<typename T>
T clamp(T v, T a, T b) {return std::min(b, std::max(a, v));}

using namespace Game;

const float VERT_BOX_SLOP=4.0f;
const float HORZ_BOX_SLOP=0.5f;

ActiveObject::ActiveObject(const ObjectTemplate *tobj, bool withColliders):
	Object(tobj, withColliders), spd(), /*sspd(),*/ mspd(), maxSpeed(tobj->getMaxSpeed()), bounced(false), movstate(MOVE_NONE), detstate(DETECTOR_NONE)
	{
	sprite.setDirection((unsigned)Engine::Sprite::DirectionName::LEFT);
	}


void ActiveObject::update(float dt)
	{
	updateMove(dt);
	updateAnim();
	Object::update(dt);

	movstate=MOVE_NONE;
	}


void ActiveObject::updateMove(float dt)
	{
	const AVector G=level->getGravity();

	spd=spd+G*dt;

	const AVector spddt=spd*dt;

	detstate=DETECTOR_NONE;

	// Slizganie
	if(cfloor && !bounced)
		{
		if(mspd.x!=0.0f)
			spd.x=mspd.x*cfloor.getFriction()+clamp(spd.x+mspd.x*cfloor.getFriction()*dt, -maxSpeed, maxSpeed)*(1.0f-cfloor.getFriction());
		else
			spd.x=mspd.x*cfloor.getFriction()+clamp(spd.x-spd.x*cfloor.getFriction()*dt, -maxSpeed, maxSpeed)*(1.0f-cfloor.getFriction());

		if(cfloor.getFriction()==0.0f && spd.x<maxSpeed*0.2f && spd.x>-maxSpeed*0.2f)
			spd.x+=mspd.x*dt;
		}

	// W poziomie
	if(spddt.x!=0.0f)
		{
		Box dhorz=(spddt.x>0.0f)?getDetectorRight(VERT_BOX_SLOP):getDetectorLeft(VERT_BOX_SLOP);

		const int dirx=(spddt.x>0.0f)?1:-1;
		const int ipart=dirx*spddt.x;
		const float fpart=spddt.x-(int)(spddt.x);

		dhorz.pos.x+=fpart;

		Collider collider=level->test(Box(dhorz.pos+AVector(fpart, 0), dhorz.size));

		if(!collider)
			{
			pos.x+=fpart;
			dhorz.pos.x+=fpart;

			for(int i=0; i<ipart; ++i)
				{
				collider=level->test(dhorz);

				if(!collider)
					{
					pos.x+=dirx;
					dhorz.pos.x+=dirx;

					continue;
					}

				pos.x+=dirx*(collider.intersection(dhorz).x+HORZ_BOX_SLOP);

				detstate|=(dirx==1)?DETECTOR_RIGHT:DETECTOR_LEFT;
				//movstate&=~(MOVE_LEFT|MOVE_RIGHT);

				break;
				}
			}
		else
			{
			pos.x+=dirx*(collider.intersection(dhorz).x+HORZ_BOX_SLOP);

			detstate|=(dirx==1)?DETECTOR_RIGHT:DETECTOR_LEFT;
			//movstate&=~(MOVE_LEFT|MOVE_RIGHT);
			}
		}

	// W pionie
		{
		Box dvert=(spddt.y>0.0f)?getDetectorTop():getDetectorBottom();

		const int diry=(spddt.y>0.0f)?1:-1;
		const int ipart=diry*spddt.y;
		const float fpart=spddt.y-(int)(spddt.y);

		dvert.pos.y+=fpart;

		Collider collider=level->test(Box(dvert.pos+AVector(0, fpart), dvert.size));

		if(!collider)
			{
			pos.y+=fpart;
			dvert.pos.y+=fpart;

			for(int i=0; i<ipart; ++i)
				{
				collider=level->test(dvert);

				if(!collider)
					{
					pos.y+=diry;
					dvert.pos.y+=diry;

					continue;
					}

				spd.y=0.0f;
				pos.y+=diry*(collider.intersection(dvert).y+HORZ_BOX_SLOP);

				detstate|=(diry==1)?DETECTOR_TOP:DETECTOR_BOTTOM;

				break;
				}
			}
		else
			{
			spd.y=0.0f;
			pos.y+=diry*(collider.intersection(dvert).y+HORZ_BOX_SLOP);

			detstate|=(diry==1)?DETECTOR_TOP:DETECTOR_BOTTOM;
			}
		}

	// Zatrzymywanie
	if(!bounced && (detstate&DETECTOR_BOTTOM) && (detstate&(DETECTOR_LEFT|DETECTOR_RIGHT)))
		spd.x=0.0f;

	if(detstate&DETECTOR_BOTTOM)
		bounced=false;

	// Koncowe
	if(spd.y>0.0f)
		movstate|=MOVE_JUMP;
	else if(spd.y<0.0f)
		movstate|=MOVE_FALL;
	else if((spd.x<-1.0f || spd.x>1.0f) && !(movstate&(MOVE_LEFT|MOVE_RIGHT)))
		movstate|=MOVE_SLIDE;

	cfloor=level->test(getDetectorBottom());

	if(detstate&DETECTOR_BOTTOM)
		mspd=AVector();
	}

void ActiveObject::updateAnim()
	{
	if(movstate&MOVE_LEFT)
		{
		sprite.setDirection((unsigned)Engine::Sprite::DirectionName::LEFT);
		}
	else if(movstate&MOVE_RIGHT)
		{
		sprite.setDirection((unsigned)Engine::Sprite::DirectionName::RIGHT);
		}

	if(movstate&MOVE_JUMP)
		{
		if((spd.x<0.0f && sprite.getDirectionID()==(unsigned)Engine::Sprite::DirectionName::RIGHT) ||
		   (spd.x>0.0f && sprite.getDirectionID()==(unsigned)Engine::Sprite::DirectionName::LEFT))
			sprite.setAnimation("falling-backward");
		else
			sprite.setAnimation("jumping");
		}
	else if(movstate&MOVE_FALL)
		{
		if((spd.x<0.0f && sprite.getDirectionID()==(unsigned)Engine::Sprite::DirectionName::RIGHT) ||
		   (spd.x>0.0f && sprite.getDirectionID()==(unsigned)Engine::Sprite::DirectionName::LEFT))
			sprite.setAnimation("falling-backward");
		else
			sprite.setAnimation("falling-forward");
		}
	else if(movstate&MOVE_SLIDE)
		{
		sprite.setAnimation("sliding");
		}
	else if(movstate&(MOVE_RIGHT|MOVE_LEFT))
		//(movstate&MOVE_RIGHT && !(detstate&DETECTOR_RIGHT))||
		//(movstate&MOVE_LEFT && !(detstate&DETECTOR_LEFT))
		{
		sprite.setAnimation("walking");
		}
	else
		{
		sprite.setAnimation("standing");
		}
	}

void ActiveObject::print()
	{
	Object::print();

	/*Box dett=getDetectorTop();
	if(detstate&DETECTOR_TOP)
		Engine::Render::setColor(AVector(1, 0.5f, 0, 1));
	else
		Engine::Render::setColor(AVector(0, 1, 0, 1));
	Engine::Render::drawRectLine(dett.pos+AVector(0, 0, 3), dett.pos+dett.size);

	Box detb=getDetectorBottom();
	if(detstate&DETECTOR_BOTTOM)
		Engine::Render::setColor(AVector(1, 0.5f, 0, 1));
	else
		Engine::Render::setColor(AVector(0, 1, 0, 1));
	Engine::Render::drawRectLine(detb.pos+AVector(0, 0, 3), detb.pos+detb.size);

	Box detl=getDetectorLeft(VERT_BOX_SLOP);
	if(detstate&DETECTOR_LEFT)
		Engine::Render::setColor(AVector(1, 0.5f, 0, 1));
	else
		Engine::Render::setColor(AVector(0, 1, 0, 1));
	Engine::Render::drawRectLine(detl.pos+AVector(0, 0, 3), detl.pos+detl.size);

	Box detr=getDetectorRight(VERT_BOX_SLOP);
	if(detstate&DETECTOR_RIGHT)
		Engine::Render::setColor(AVector(1, 0.5f, 0, 1));
	else
		Engine::Render::setColor(AVector(0, 1, 0, 1));
	Engine::Render::drawRectLine(detr.pos+AVector(0, 0, 3), detr.pos+detr.size);


	Engine::Render::setColor(AVector(1, 1, 1, 1));*/
	}


void ActiveObject::moveLeft()
	{
	if(!(detstate&DETECTOR_BOTTOM))
		return;

	mspd.x=-maxSpeed;

	movstate=(movstate|MOVE_LEFT)&(~MOVE_RIGHT);
	}

void ActiveObject::moveRight()
	{
	if(!(detstate&DETECTOR_BOTTOM))
		return;
	if(this->getTemplate()->getName()=="player_male")
		{
		nop();
		}


	mspd.x= maxSpeed;

	movstate=(movstate|MOVE_RIGHT)&(~MOVE_LEFT);
	}

void ActiveObject::jump(float height)
	{
	// XXX Dopoki wcisniety przycisk skoku nie zeruj predkosci w Y?

	// 1. s=at^2/2 => t=sqrt(2s/a)
	// 2. v=at
	// 3. => v=a sqrt(2s/a) => v=sqrt(a^2 2s/a) => v=sqrt(2sa)

	if(!(detstate&DETECTOR_BOTTOM))
		return;

	const AVector vG=level->getGravity();
	const float G=AVectorLength(vG).w;
	const AVector nG=vG/G;

	spd=spd-nG*sqrt(2.0f*height*G);
	}

void ActiveObject::bounceLeft(float height, float distance)
	{
	const AVector vG=level->getGravity();
	const AVector vTG=vG^AVector(0, 0, 1);
	const float G=AVectorLength(vG).w;
	const AVector nG=vG/G;
	const AVector nTG=vTG/G;

	spd=-nG*sqrt(2.0f*height*G) + nTG*distance*0.5f/sqrt(2.0f*height/G);

	movstate|=MOVE_RIGHT;
	bounced=true;
	}

void ActiveObject::bounceRight(float height, float distance)
	{
	const AVector vG=level->getGravity();
	const AVector vTG=vG^AVector(0, 0, 1);
	const float G=AVectorLength(vG).w;
	const AVector nG=vG/G;
	const AVector nTG=vTG/G;

	spd=-nG*sqrt(2.0f*height*G) - nTG*distance*0.5f/sqrt(2.0f*height/G);

	movstate|=MOVE_LEFT;
	bounced=true;
	}
