/*
 * ativeobject.h
 *
 *  Created on: 1 maj 2016
 *      Author: crm
 */

#pragma once

#include "object.h"

namespace Game
	{
	class ActiveObject: public Object
		{
		protected:
			enum MovementState
				{
				MOVE_NONE=          0x0,
				MOVE_LEFT=          0x1<<0,
				MOVE_RIGHT=         0x1<<1,
				MOVE_JUMP=          0x1<<2,
				MOVE_FALL=          0x1<<3,
				MOVE_SLIDE=         0x1<<4,
				};

			enum DetectorState
				{
				DETECTOR_NONE=      0x0,
				DETECTOR_LEFT=      0x1<<0,
				DETECTOR_RIGHT=     0x1<<1,
				DETECTOR_TOP=       0x1<<2,
				DETECTOR_BOTTOM=    0x1<<3,
				};

			AVector spd;  // Szybkosc
			//AVector sspd; // Slizganie
			AVector mspd; // Szybkosc samodzielnie sobie nadana

			float maxSpeed;
			bool bounced;

			Collider cfloor;

			unsigned movstate;
			unsigned detstate;

			inline Box getDetectorLeft(float offset=0.0f) const {const Box bb=getBoundingBox(); return Box(bb.pos+AVector(-1, offset), AVector(1, bb.size.y-offset));}
			inline Box getDetectorRight(float offset=0.0f) const {const Box bb=getBoundingBox(); return Box(bb.pos+AVector(bb.size.x, offset), AVector(1, bb.size.y-offset));}
			inline Box getDetectorTop() const {const Box bb=getBoundingBox(); return Box(bb.pos+AVector(0, bb.size.y), AVector(bb.size.x, 1));}
			inline Box getDetectorBottom() const {const Box bb=getBoundingBox(); return Box(bb.pos+AVector(2, 1), AVector(bb.size.x-4, 1));}
			inline Box getDetectorCenter() const {const Box bb=getBoundingBox(); return Box(bb.pos+AVector(2, 1), AVector(bb.size.x-4, bb.size.y-2));}

			void updateMove(float dt);
			void updateAnim();

		public:
			ActiveObject(): Object(), spd(), /*sspd(),*/ mspd(), maxSpeed(0.0f), bounced(false), movstate(MOVE_NONE), detstate(DETECTOR_NONE) {}
			ActiveObject(const ObjectTemplate *tobj, bool withColliders=true);

			virtual void update(float dt);
			virtual void print();

			virtual char* serialize(int &size)=0;
			virtual void deserialize(char *buff)=0;

			// Sterowanie
			void moveLeft();
			void moveRight();
			void jump(float height);
			void bounceLeft(float height, float distance);
			void bounceRight(float height, float distance);

			// Sety
			void setSpeed(const AVector& s) {spd=s;}

			// Gety
			AVector getSpeed() const {return spd;}
			//unsigned getMovementState() const {return movstate;}
			//unsigned getDetectorState() const {return detstate;}
		};
	}
