/*
 * dialogue.cpp
 *
 *  Created on: 4 maj 2016
 *      Author: crm
 */

#include "dialogue.h"

#include "log.h"
#include "ini.h"
#include "parser.h"

#include "game.h"

using namespace Game;

bool Dialogue::init(const std::string& path)
	{
	if(this->path==path)
		{
		Log::info("Dialogue.init: Skrytp \"%s\" juz zaladowany", path.c_str());

		setMode(Mode::NORMAL);
		setNode(1);
		wait=0.0f;

		return true;
		}

	clear();

	Ini ini;

	Log::info("Dialogue.init: Wczytywanie skryptu \"%s\"", path.c_str());

	if(!ini.open(path, "dialog"))
		{
		Log::fail("Dialogue.init: Nie udalo sie otworzyc pliku \"%s\"", path.c_str());
		return false;
		}

	Parser pars("", " \t");

	std::string key;
	std::string val;

	bool end=!ini.getFirst(key, val);

	unsigned icount=0u;
	unsigned ncount=0u;

	while(!end)
		{
		pars=key;

		int keyNodeID=0u;
		int keyItemID=0u;

		if(pars.count()<1 || !pars.isVal(0))
			{
			Log::fail("Dialogue.init: Zly klucz w pliku ini: \"%s\"", key.c_str());
			end=!ini.getNext(key, val);
			continue;
			}

		keyNodeID=pars.val(0);

		if(pars.count()>1)
			keyItemID=pars.val(1);

		Log::debug("Dialogue.init: node: %u, item: %u, value: \"%s\"", keyNodeID, keyItemID, val.c_str());

		if(nodes.find(keyNodeID)==nodes.end())
			{
			Log::debug("Dialogue.init: Dodawanie node: %u", keyNodeID);
			nodes[keyNodeID]=ScriptNode();
			nodes[keyNodeID].id=keyNodeID;

			++ncount;
			}

		ScriptNode& node=nodes[keyNodeID];

		if(node.items.find(keyItemID)==node.items.end())
			{
			Log::debug("Dialogue.init: Dodawanie item: %u", keyItemID);
			node.items[keyItemID]=ScriptItem();
			node.items[keyItemID].idx=keyItemID;

			++icount;
			}
		else
			{
			Log::fail("Dialogue.init: Konflikt: Komenda [%u_%u] juz ustawiona", keyNodeID, keyItemID);
			}

		ScriptItem& item=node.items[keyItemID];

		item.text=val;

		end=!ini.getNext(key, val);
		}

	ini.clear();

	Log::ok("Dialogue.init: Wczytano skrypt \"%s\"", path.c_str());
	Log::debug("Dialogue.init: Skrypt \"%s\"; nodes: %u; items: %u", path.c_str(), ncount, icount);

	this->path=path;

	setMode(Mode::NORMAL);
	setNode(1);
	wait=0.0f;

	return true;
	}

void Dialogue::update(float dt)
	{
	if(mode==Mode::NONE)
		return;

	if(wait>0.0f)
		{
		wait-=dt;

		toUpdate=true;
		}
	else if(toUpdate)
		{
		toUpdate=false;
		update();

		if(mode==Mode::CHOICE_SMALL || mode==Mode::CHOICE_BIG)
			{
			updateChoice();
			}
		}

	if(nodeID<=0)
		{
		setMode(Mode::NONE);
		}
	}

void Dialogue::clear()
	{
	path="";

	toUpdate=true;

	mode=Mode::NONE;
	nodeID=0;
	itemID=0;

	nodes.clear();

	wait=0.0f;

	message="";

	choice=0u;
	choices.clear();
	}


void Dialogue::setMode(Mode s)
	{
	toUpdate=true;

	if(mode==s)
		return;

	mode=s;

	/*if(mode==Mode::NONE)
		{
		game->getFade().set(0, (Engine::SpritePtr)"sprite/fade_dialogue.xml", 0.0f);
		}
	else
		{
		game->getFade().set(0, (Engine::SpritePtr)"sprite/fade_dialogue.xml", 1.0f);
		}*/
	}


void Dialogue::next()
	{
	if(mode==Mode::CHOICE_BIG || mode==Mode::CHOICE_SMALL)
		return confirm();
	if(mode!=Mode::NORMAL)
		return;
	if(toUpdate)
		return;
	if(wait>0.0f)
		return;

	setNode(nodeID-1);
	}

void Dialogue::confirm()
	{
	if(mode!=Mode::CHOICE_BIG && mode!=Mode::CHOICE_SMALL)
		return;
	if(toUpdate)
		return;
	if(wait>0.0f)
		return;

	Parser pars(choices[choice]->text, " \t");

	mode=Mode::NORMAL;

	choices.clear();

	setNode(pars.val(1));
	}

void Dialogue::choicePrevious()
	{
	if(mode!=Mode::CHOICE_BIG && mode!=Mode::CHOICE_SMALL)
		return;
	if(choices.empty())
		return;

	if(choice==0u)
		choice=choices.size()-1u;
	else
		--choice;

	updateChoice();
	}

void Dialogue::choiceNext()
	{
	if(mode!=Mode::CHOICE_BIG && mode!=Mode::CHOICE_SMALL)
		return;
	if(choices.empty())
		return;

	++choice;

	if(choice>=choices.size())
		choice=0u;

	updateChoice();
	}



void Dialogue::update()
	{
	Parser pars("", " \t");

	int watchdog=0;

	for(/**/; itemID<100; ++itemID)
		{
		++watchdog;

		if(watchdog>200)
			{
			Log::fail("Dialogue.update: Skrypt sie zawiesil, przerywanie");
			toUpdate=true;
			break;
			}

		if(nodes.find(nodeID)==nodes.end())
			return;

		ScriptNode &node=nodes[nodeID];

		if(node.items.find(itemID)==node.items.end())
			continue;

		ScriptItem &item=node.items[itemID];

		//Log::debug("Dialogue.update[%4u:%02u]: \"%s\"", nodeID, itemID, item.text.c_str());

		pars=item.text;

		if(pars.count()<1)
			continue;

		/*****************************************************************************/
		/***************** Wypisywanie tekstu ****************************************/
		/*****************************************************************************/
		if(pars[0]=="str")
			{
			pars.del(0);
			message+=pars.getText();
			}
		else if(pars[0]=="strnl")
			{
			pars.del(0);
			message+=pars.getText()+"\n";
			}
		else if(pars[0]=="nl")
			{
			message+="\n";
			}
		else if(pars[0]=="clear")
			{
			message="";
			}
		else if(pars[0]=="strval")
			{
			if(pars.count()<2)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"strval\", oczekiwano 2, otrzymano %d", pars.count());
				continue;
				}

			char tmp[16];

			sprintf(tmp, "%d", game->getDatabase().getVal(pars[1]));

			message+=tmp;
			}
		/*****************************************************************************/
		/***************** Przeskakiwanie miedzy komendami ***************************/
		/*****************************************************************************/
		else if(pars[0]=="setb")
			{
			setNode(pars.val(1));
			setItem(-1);
			continue;
			}
		else if(pars[0]=="seti")
			{
			setItem(pars.val(1)-1);
			continue;
			}
		else if(pars[0]=="w8")
			{
			if(pars.count()<2)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"w8\", oczekiwano 2, otrzymano %d", pars.count());
				continue;
				}

			wait=pars.valFloat(1);
			setItem(itemID+1);
			break;
			}
		else if(pars[0]=="stop")
			{
			break;
			}
		/*****************************************************************************/
		/***************** Zmiana skryptu ********************************************/
		/*****************************************************************************/
		else if(pars[0]=="npc")
			{
			if(pars.count()<2)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"npc\", oczekiwano 2, otrzymano %d", pars.count());
				continue;
				}

			if(!init(pars[2]))
				{
				Log::fail("Dialogue.update: \"npc\": Nie udalo sie otworzyc zaladowac skryptu \"%s\"", pars[2].c_str());
				break;
				}

			setNode(pars.val(1));
			setItem(-1);
			continue;
			}
		/*****************************************************************************/
		/***************** Obsługa wartości ******************************************/
		/*****************************************************************************/
		else if(pars[0]=="save")
			{
			if(pars.count()<4)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"set\", oczekiwano 4, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				Log::debug("Dialogue.update: \"set\": ustawianie \"%s\" na %d", pars[2].c_str(), pars.val(3));
				game->getDatabase().setVal(pars[2], pars.val(3));
				}
			else
				{
				const std::string key=pars[2];

				pars.del(2);
				pars.del(1);
				pars.del(0);

				game->getDatabase().setStr(key, pars.getText());
				}
			}
		else if(pars[0]=="add")
			{
			if(pars.count()<4)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"add\", oczekiwano 4, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				game->getDatabase().setVal(pars[2], game->getDatabase().getVal(pars[2])+pars.val(3));
				Log::debug("Dialogue.update: \"add\": Dodawanie %d do \"%s\": %d", pars.val(3), pars[2].c_str(), game->getDatabase().getVal(pars[2]));
				}
			else
				{
				const std::string key=pars[2];

				pars.del(2);
				pars.del(1);
				pars.del(0);

				game->getDatabase().setStr(key, game->getDatabase().getStr(key)+pars.getText());
				}
			}
		else if(pars[0]=="sub")
			{
			if(pars.count()<4)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"sub\", oczekiwano 4, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				game->getDatabase().setVal(pars[2], game->getDatabase().getVal(pars[2])-pars.val(3));
				Log::debug("Dialogue.update: \"add\": Odejmowanie %d od \"%s\": %d", pars.val(3), pars[2].c_str(), game->getDatabase().getVal(pars[2]));
				}
			else
				{
				Log::fail("Dialogue.update: \"sub\": TODO odejmowanie tekstow (substring)");
				}
			}
		else if(pars[0]=="del")
			{
			if(pars.count()<3)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"del\", oczekiwano 3, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				game->getDatabase().setVal(pars[2], 0);
				}
			else
				{
				game->getDatabase().setStr(pars[2], "");
				}
			}
		/*****************************************************************************/
		/***************** Wybory ****************************************************/
		/*****************************************************************************/
		else if(pars[0]=="wyb")
			{
			if(pars.count()<2)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"wyb\", oczekiwano 2, otrzymano %d", pars.count());
				continue;
				}

			if(mode!=Mode::CHOICE_BIG && mode!=Mode::CHOICE_SMALL)
				{
				choice=0;
				choices.clear();
				mode=Mode::CHOICE_SMALL;
				}

			choices.push_back(&item);
			}
		/*****************************************************************************/
		/***************** Instrukcje warunkowe **************************************/
		/*****************************************************************************/
		else if(pars[0]=="if")
			{
			if(pars.count()<6)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"if\", oczekiwano 6/7, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				int dbv=game->getDatabase().getVal(pars[2]);

				if(pars.count()==6)
					{
					if(dbv==pars.val(3))
						{
						Log::debug("Dialogue.update: \"if\": Porownywanie z \"%s\": %s==%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setNode(pars.val(4));
						setItem(-1);
						continue;
						}
					else
						{
						Log::debug("Dialogue.update: \"if\": Porownywanie z \"%s\": %s!=%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setNode(pars.val(5));
						setItem(-1);
						continue;
						}
					}
				else
					{
					int pv=pars.val(3);

					if(dbv<pv)
						{
						Log::debug("Dialogue.update: \"if\": Porownywanie z \"%s\": %s<%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setNode(pars.val(4));
						setItem(-1);
						continue;
						}
					else if(dbv==pv)
						{
						Log::debug("Dialogue.update: \"if\": Porownywanie z \"%s\": %s==%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setNode(pars.val(5));
						setItem(-1);
						continue;
						}
					else
						{
						Log::debug("Dialogue.update: \"if\": Porownywanie z \"%s\": %s>%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setNode(pars.val(6));
						setItem(-1);
						continue;
						}
					}
				}
			else
				{
				Log::fail("Dialogue.update: TODO \"if\": Porownywanie stringow");
				}
			}
		else if(pars[0]=="ifi")
			{
			if(pars.count()<6)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"ifi\", oczekiwano 6/7, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				int dbv=game->getDatabase().getVal(pars[2]);

				if(pars.count()==6)
					{
					if(dbv==pars.val(3))
						{
						Log::debug("Dialogue.update: \"ifi\": Porownywanie z \"%s\": %s==%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setItem(pars.val(4)-1);
						continue;
						}
					else
						{
						Log::debug("Dialogue.update: \"ifi\": Porownywanie z \"%s\": %s!=%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setItem(pars.val(5)-1);
						continue;
						}
					}
				else
					{
					int pv=pars.val(3);

					if(dbv<pv)
						{
						Log::debug("Dialogue.update: \"ifi\": Porownywanie z \"%s\": %s<%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setItem(pars.val(4)-1);
						continue;
						}
					else if(dbv==pv)
						{
						Log::debug("Dialogue.update: \"ifi\": Porownywanie z \"%s\": %s==%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setItem(pars.val(5)-1);
						continue;
						}
					else
						{
						Log::debug("Dialogue.update: \"ifi\": Porownywanie z \"%s\": %s>%d", pars[2].c_str(), pars[3].c_str(), dbv);
						setItem(pars.val(6)-1);
						continue;
						}
					}
				}
			else
				{
				Log::fail("Dialogue.update: TODO \"switch\": Porownywanie stringow");
				}
			}
		else if(pars[0]=="switch")
			{
			if(pars.count()<6)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"ifi\", oczekiwano 6/7, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				int dbv=game->getDatabase().getVal(pars[2]);

				if(dbv<0 || dbv>pars.count()-3)
					{
					Log::debug("Dialogue.update: \"switch\": Brak akcji dla wartosci %d (lub jest ona ujemna)", dbv);
					continue;
					}

				Log::debug("Dialogue.update: \"switch\": Akcja %d -> %d", dbv, pars.val(dbv+3));

				setNode(pars.val(dbv+3));
				setItem(itemID-1);
				continue;
				}
			else
				{
				Log::fail("Dialogue.update: TODO \"switch\": Porownywanie stringow");
				}
			}
		else if(pars[0]=="switchi")
			{
			if(pars.count()<6)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"switchi\", oczekiwano 6/7, otrzymano %d", pars.count());
				continue;
				}

			if(pars[1]=="val")
				{
				int dbv=game->getDatabase().getVal(pars[2]);

				if(dbv<0 || dbv>pars.count()-3)
					{
					Log::debug("Dialogue.update: \"switchi\": Brak akcji dla wartosci %d (lub jest ona ujemna)", dbv);
					continue;
					}

				Log::debug("Dialogue.update: \"switchi\": Akcja %d -> %d", dbv, pars.val(dbv)-3);

				setItem(pars.val(dbv-3)-1);
				continue;
				}
			else
				{
				Log::fail("Dialogue.update: TODO \"switchi\": Porownywanie stringow");
				}
			}
		/*****************************************************************************/
		/***************** Kasowanie i pojawianie NPC ********************************/
		/*****************************************************************************/
		else if(pars[0]=="pwn")
			{
			if(pars.count()<3)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"pwn\", oczekiwano 3, otrzymano %d", pars.count());
				continue;
				}

			if(game->getLevel()->getID()==pars[1])
				{
				NPC *npc=game->getLevel()->getNPCByID(pars[2]);

				if(!npc)
					{
					Log::fail("Dialogue.update: Nie znaleziono NPC o ID \"%s\" w aktualnym poziomie o ID \"%s\"", pars[2].c_str(), game->getLevel()->getID().c_str());
					continue;
					}

				npc->setDisabled(true);
				}

			const std::string key=pars[1]+":"+pars[2];

			Parser npcp(game->getDatabase().getStr(key), " ");

			if(npcp.count()<1)
				{
				Log::fail("Dialogue.update: Nie znaleziono NPC \"%s\" w bazie", key.c_str());
				continue;
				}

			Log::debug("Dialogue.update: bylo: %s", npcp.getText().c_str());
			npcp.swap(0, "1");
			Log::debug("Dialogue.update: jest: %s", npcp.getText().c_str());

			game->getDatabase().setStr(key, npcp.getText());
			}
		else if(pars[0]=="res")
			{
			if(pars.count()<3)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"res\", oczekiwano 3, otrzymano %d", pars.count());
				continue;
				}

			if(game->getLevel()->getID()==pars[1])
				{
				NPC *npc=game->getLevel()->getNPCByID(pars[2]);

				if(!npc)
					{
					Log::fail("Dialogue.update: Nie znaleziono NPC o ID \"%s\" w aktualnym poziomie o ID \"%s\"", pars[2].c_str(), game->getLevel()->getID().c_str());
					continue;
					}

				npc->setDisabled(false);
				}

			const std::string key=pars[1]+":"+pars[2];

			Parser npcp(game->getDatabase().getStr(key), " ");

			if(npcp.count()<1)
				{
				Log::fail("Dialogue.update: Nie znaleziono NPC \"%s\" w bazie", key.c_str());
				continue;
				}

			Log::debug("Dialogue.update: bylo: %s", npcp.getText().c_str());
			npcp.swap(0, "0");
			Log::debug("Dialogue.update: jest: %s", npcp.getText().c_str());

			game->getDatabase().setStr(key, npcp.getText());
			}
		/*****************************************************************************/
		/***************** Obroty ****************************************************/
		/*****************************************************************************/
		else if(pars[0]=="rotate")
			{
			// Obroty NPC
			if(pars.count()>=4)
				{
				Engine::Sprite::DirectionName dn;

				if(pars[3]=="left")
					dn=Engine::Sprite::DirectionName::LEFT;
				else
					dn=Engine::Sprite::DirectionName::RIGHT;

				if(game->getLevel()->getID()==pars[1])
					{
					NPC *npc=game->getLevel()->getNPCByID(pars[2]);

					if(!npc)
						{
						Log::fail("Dialogue.update: Nie znaleziono NPC o ID \"%s\" w aktualnym poziomie o ID \"%s\"", pars[2].c_str(), game->getLevel()->getID().c_str());
						continue;
						}

					npc->getSprite().setDirection((unsigned)dn);
					}

				const std::string key=pars[1]+":"+pars[2];

				Parser npcp(game->getDatabase().getStr(key), " ");

				if(npcp.count()<8)
					{
					Log::fail("Dialogue.update: Nie znaleziono NPC \"%s\" w bazie", key.c_str());
					continue;
					}

				char tmp[8];
				sprintf(tmp, "%d", (int)dn);

				Log::debug("Dialogue.update: bylo: %s", npcp.getText().c_str());
				npcp.swap(7, tmp);
				Log::debug("Dialogue.update: jest: %s", npcp.getText().c_str());

				game->getDatabase().setStr(key, npcp.getText());
				}
			// Obroty gracza
			else if(pars.count()==2)
				{
				Engine::Sprite::DirectionName dn;

				if(pars[3]=="left")
					dn=Engine::Sprite::DirectionName::LEFT;
				else
					dn=Engine::Sprite::DirectionName::RIGHT;

				game->getLevel()->getPlayer()->getSprite().setDirection((unsigned)dn);
				}
			else
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"rotate\", oczekiwano 2/4, otrzymano %d", pars.count());
				continue;
				}
			}
		/*****************************************************************************/
		/***************** Zmiana pozycji ********************************************/
		/*****************************************************************************/
		else if(pars[0]=="pos")
			{
			// Pozycja NPC
			if(pars.count()>=5)
				{
				float x, y, z;

				x=pars.valFloat(3);
				y=pars.valFloat(4);

				if(pars.count()>5)
					z=pars.valFloat(5);
				else
					z=0.0f;

				if(game->getLevel()->getID()==pars[1])
					{
					NPC *npc=game->getLevel()->getNPCByID(pars[2]);

					if(!npc)
						{
						Log::fail("Dialogue.update: Nie znaleziono NPC o ID \"%s\" w aktualnym poziomie o ID \"%s\"", pars[2].c_str(), game->getLevel()->getID().c_str());
						continue;
						}

					npc->setPosition(AVector(x, y, z));
					}

				const std::string key=pars[1]+":"+pars[2];

				Parser npcp(game->getDatabase().getStr(key), " ");

				if(npcp.count()<8)
					{
					Log::fail("Dialogue.update: Nie znaleziono NPC \"%s\" w bazie", key.c_str());
					continue;
					}

				char tmp[8];

				Log::debug("Dialogue.update: bylo: %s", npcp.getText().c_str());

				sprintf(tmp, "%f", x);
				npcp.swap(1, tmp);
				sprintf(tmp, "%f", y);
				npcp.swap(2, tmp);
				sprintf(tmp, "%f", z);
				npcp.swap(3, tmp);

				Log::debug("Dialogue.update: jest: %s", npcp.getText().c_str());

				game->getDatabase().setStr(key, npcp.getText());
				}
			// Pozycja gracza
			else if(pars.count()==2 || pars.count()==3)
				{
				float x, y, z;

				x=pars.valFloat(1);
				y=pars.valFloat(2);

				if(pars.count()>3)
					z=pars.valFloat(3);
				else
					z=0.0f;

				game->getLevel()->getPlayer()->setPosition(AVector(x, y, z));
				game->getCamera().setPos(game->getPlayer()->getPosition());
				}
			else
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"rotate\", oczekiwano 2/4, otrzymano %d", pars.count());
				continue;
				}
			}
		else if(pars[0]=="goto")
			{
			if(pars.count()<2)
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"goto\", oczekiwano 2/4/6, otrzymano %d", pars.count());
				continue;
				}

			if(pars.count()>=2)
				{
				if(game->getLevel()->getID()==pars[1])
					{
					Log::info("Dialogue.update: Przeskok do aktualnego poziomu");
					}
				else if(!game->setLevel(pars[1]))
					{
					Log::fail("Dialogue.update: Nie udalo sie wczytac poziomu \"%s\"", pars[1].c_str());
					continue;
					}
				}

			if(pars.count()>=4)
				{
				game->getPlayer()->setPosition(AVector(pars.valFloat(2), pars.valFloat(3), 0));

				game->getCamera().setPos(game->getPlayer()->getPosition());
				}

			if(pars.count()>=6)
				{
				if(!init(pars[4]))
					{
					Log::fail("Dialogue.update: Nie udalo sie wczytac skryptu \"%s\"", pars[5].c_str());
					break;
					}

				setNode(pars.val(5));
				setItem(-1);

				game->getPlayer()->setPlayerModeTalking();
				}
			else
				{
				setNode(0);
				game->getPlayer()->setPlayerModeNormal();
				}
			}
		/*****************************************************************************/
		/***************** Animacje **************************************************/
		/*****************************************************************************/
		else if(pars[0]=="anim")
			{
			// Animacje NPC
			if(pars.count()>=4)
				{
				if(game->getLevel()->getID()==pars[1])
					{
					NPC *npc=game->getLevel()->getNPCByID(pars[2]);

					if(!npc)
						{
						Log::fail("Dialogue.update: Nie znaleziono NPC o ID \"%s\" w aktualnym poziomie o ID \"%s\"", pars[2].c_str(), game->getLevel()->getID().c_str());
						continue;
						}

					npc->setSprite(pars[3]);
					}

				const std::string key=pars[1]+":"+pars[2];

				Parser npcp(game->getDatabase().getStr(key), " ");

				if(npcp.count()<8)
					{
					Log::fail("Dialogue.update: Nie znaleziono NPC \"%s\" w bazie", key.c_str());
					continue;
					}

				Log::debug("Dialogue.update: bylo: %s", npcp.getText().c_str());
				npcp.swap(5, pars[3]);
				Log::debug("Dialogue.update: jest: %s", npcp.getText().c_str());

				game->getDatabase().setStr(key, npcp.getText());
				}
			// Animacja gracza
			else if(pars.count()==2)
				{
				game->getLevel()->getPlayer()->setSprite(pars[1]);
				}
			else
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"rotate\", oczekiwano 2/4, otrzymano %d", pars.count());
				continue;
				}
			}
		/*****************************************************************************/
		/***************** Dzwieki ***************************************************/
		/*****************************************************************************/
		else if(pars[0]=="sound")
			{
			game->getSound().play(pars[1].c_str(), -1, 1, 0, 0.0f, 1.0f);
			}
		else if(pars[0]=="music")
			{
			// TODO
			Log::info("TODO");
			}
		/*****************************************************************************/
		/***************** Lua *******************************************************/
		/*****************************************************************************/
		else if(pars[0]=="lua")
			{
			pars.del(0);

			Log::debug("Dialogue.update: \"lua\": \"%s\"", pars.getText().c_str());

			game->getLevel()->getLua().parse(pars.getText());
			}
		/*****************************************************************************/
		/***************** Fade ******************************************************/
		/*****************************************************************************/
		else if(pars[0]=="fade")
			{
			if(pars.count()==2)
				{
				game->getFade().set(1, pars.valFloat(2));
				}
			else if(pars.count()==3)
				{
				game->getFade().set(1, pars[1], pars.valFloat(2));
				}
			else if(pars.count()==4)
				{
				game->getFade().set(1, pars[1], pars.valFloat(2), pars.valFloat(3));
				}
			else
				{
				Log::fail("Dialogue.update: Za malo argumentow do \"goto\", oczekiwano 2/4/6, otrzymano %d", pars.count());
				continue;
				}
			}
		else if(pars[0]=="unfade")
			{
			game->getFade().set(1, (Engine::SpritePtr)"", 0.0f, 0.0f);
			}
		else if(pars[0]=="scene")
			{
			game->getFade().set(0, (Engine::SpritePtr)"sprite/fade_dialogue.xml", 1.0f);
			}
		else if(pars[0]=="unscene")
			{
			game->getFade().set(0, (Engine::SpritePtr)"sprite/fade_dialogue.xml", 0.0f);
			}
		else if(pars[0]=="dbreset")
			{
			game->getDatabase().clear();
			}
		/*****************************************************************************/
		/***************** Bledna komenda ********************************************/
		/*****************************************************************************/
		else
			{
			Log::fail("Dialogue.update: Nieprawidlowa komenda: \"%s\"", pars[0].c_str());
			}
		}
	}

void Dialogue::updateChoice()
	{
	message="";

	Parser pars("", " \t");

	for(unsigned i=0; i<choices.size(); ++i)
		{
		ScriptItem &item=*choices[i];

		pars=item.text;

		pars.del(1);
		pars.del(0);

		if(i==choice)
			message+="["+pars.getText()+"]\n";
		else
			message+=pars.getText()+"\n";
		}
	}

