/*
 * dialogue.h
 *
 *  Created on: 4 maj 2016
 *      Author: crm
 */

#pragma once

#include <string>
#include <vector>
#include <map>

#include "database.h"

#include "level.h"

namespace Game
	{
	class Dialogue
		{
		public:
			enum class Mode
				{
				NONE,
				NORMAL,
				CHOICE_SMALL,
				CHOICE_BIG,
				};

		private:
			struct ScriptItem
				{
				ScriptItem(): idx(0u), text("") {}

				int idx;

				std::string text;
				};

			struct ScriptNode
				{
				ScriptNode(): id(0u) {}
				~ScriptNode() {items.clear();}

				const ScriptItem& getItem(unsigned idx)
					{
					return items[idx];
					}

				int id;

				std::map<int, ScriptItem> items;
				};

			std::string path;

			Game *game;

			bool toUpdate;

			Mode mode;  // A
			int nodeID; // B
			int itemID; // index

			std::map<int, ScriptNode> nodes;

			float wait;    // Oczekiwanie

			std::string message;

			unsigned choice;
			std::vector<ScriptItem*> choices;

			void update();
			void updateChoice();

		public:
			Dialogue(): path(""), game(nullptr), toUpdate(true), mode(Mode::NONE), nodeID(0), itemID(0), wait(0.0f), message(""), choice(0u)
				{
				//
				}

			~Dialogue()
				{
				clear();
				}

			bool init(const std::string& path);
			void update(float dt);
			void clear();

			void next();
			void confirm();
			void choicePrevious();
			void choiceNext();

			Mode getMode() const {return mode;}
			int getNodeID() const {return nodeID;}
			int getItemID() const {return itemID;}
			std::string getMessage() const {return message;}
			bool isReady() const {return !toUpdate;}

			void setGame(Game *s) {game=s;}
			void setMode(Mode s);// {mode=s; toUpdate=true;}
			void setNode(int s) {nodeID=s; itemID=0u; message=""; toUpdate=true;}
			void setItem(int s) {itemID=s; toUpdate=true;}
		};
	}
