/*
 * main.cpp
 *
 *  Created on: 1 mar 2016
 *      Author: crm
 */
/* TODO:
 *  + Poziom m2p1 - zrobic klocek ziemi wielkości 64x64
 *  + Scena z narratorem
 *  + Zakończenie gry
 *  - Ambient (ptaszki)
 *  + Drzwi do doliny zniknęły
 */
#include <ctime>
#include <cmath>
#include <cstdlib>
#include <SDL/SDL.h>

#include "resource.h"
#include "database.h"
#include "log.h"

#include "consts.h"
#include "engine/application.h"
#include "engine/render/imagemanager.h"
#include "engine/render/spritemanager.h"
#include "engine/render/render.h"
#include "engine/sound/sound.h"
#include "engine/application.h"

#include "engine/ui/text.h"
#include "game/game.h"
#include "game/controller.h"

SDL_Event e;

Uint8 *key=SDL_GetKeyState(NULL);

Engine::Application application;

#undef main

int main(int argc, char *argv[]/*, char *env[]*/)
	{
#ifdef BUILD_RELEASE
	Log::setOut(fopen("log.txt", "w+"));
	Log::setLogLvl(Log::INFO|Log::FAIL);
#else
	Log::setLogLvl(Log::ALL|Log::DEBUG);
#endif

	Log::info("Start");
	Log::info("Build %s %s", __DATE__, __TIME__);

		{
		int s=time(NULL);
		srand(s);

		Log::debug("Seed: 0x%08X", s);
		}

	/*for(int i=0; env[i]; ++i)
		Log::info("env[%4d]: %s", i, env[i]);*/

	if(!Resource::includeList(RESLIST))
		return __LINE__;

	if(!application.init())
		return __LINE__;

	if(!Engine::Render::init(SCREEN_WIDTH, SCREEN_HEIGHT, GAME_NAME, "icon.bmp", true))
		return __LINE__;

	// Loading~
		{
		Engine::UI::Text loading("font/font_00.spr", "Uruchamianie...", Engine::Render::getScreenSize().x, Engine::Render::getScreenSize().y);
		loading.setAlignCenter();
		loading.setAlignMiddle();
		loading.update();
		Engine::Render::setMode(Engine::Render::MODE_GUI);
		Engine::Render::setColor(AVector(0, 0, 0, 1));
		Engine::Render::drawQuad(AVector(0, 0), AVector(0, Engine::Render::getScreenSize().y), AVector(Engine::Render::getScreenSize().x, Engine::Render::getScreenSize().y), AVector(Engine::Render::getScreenSize().x, 0));
		Engine::Render::setColor(AVector(1, 1, 1, 1));
		loading.print(Engine::Orientation(AVector(0, Engine::Render::getScreenSize().y, 0)));
		Engine::Render::update();

		//SDL_Delay(500);
		}

	Log::info("init: %dms", SDL_GetTicks());
	Log::info("------------------------------------------------------------");

	Game::Game game;
	game.init();
	//if(!game.setLevel("level/test.xml"))
	if(!game.setLevel("level/01_miedzyczas.xml"))
	//if(!game.setLevel("level/02_m1.xml"))
	//if(!game.setLevel("level/03_m2_p1.xml"))
	//if(!game.setLevel("level/04_m2_p2.xml"))
	//if(!game.setLevel("level/05_m2_p3.xml"))
	//if(!game.setLevel("level/06_m3_p1.xml"))
	//if(!game.setLevel("level/07_m3_p2.xml"))
	//if(!game.setLevel("level/08_m3_p3.xml"))
		return 1;

	game.getFade().set(1, (Engine::SpritePtr)"sprite/fade.xml", 0.0f, 1.0f);

#ifndef BUILD_RELEASE
	Engine::UI::Text info("font/font_00.spr", "[Prosze czekac...]", Engine::Render::getScreenSize().x, 128);
	info.update();

	unsigned fpsTimer=SDL_GetTicks();
	unsigned fpsFrames=0u;
	float lfps=-1.0f;
	float ltime=-1.0f;
#endif

	Log::info("load: %dms", SDL_GetTicks());
	Log::info("Uruchamianie petli glownej...");
	Log::info("------------------------------------------------------------");

	application.update();

#ifndef BUILD_RELEASE
	bool relonce=true;
	bool resonce=true;
	bool brkonce=true;
#endif

	while(application.running())
		{
		application.update();

		float delta=application.delta();

		/**** Update ****/
		game.update(delta);
		game.print();

#ifndef BUILD_RELEASE
		if(SDL_GetKeyState(nullptr)[SDLK_F12])
			{
			if(!resonce)
				{
				Log::info("Restartowanie poziomu i czyszczenie bazy");
				game.getDatabase().clear();
				game.setLevel(game.getLevel()->getPath());
				}

			resonce=true;
			}
		else
			resonce=false;

		if(SDL_GetKeyState(nullptr)[SDLK_F10])
			{
			if(!relonce)
				{
				Log::info("Restartowanie poziomu");
				game.setLevel(game.getLevel()->getPath());
				}

			relonce=true;
			}
		else
			relonce=false;

		if(SDL_GetKeyState(nullptr)[SDLK_F11])
			{
			if(!brkonce)
				{
				Log::info("Przerywanie dialogu");
				game.getDialogue().setNode(0);
				}

			brkonce=true;
			}
		else
			brkonce=false;
#endif

		/**** GUI ****/
#ifndef BUILD_RELEASE
			{
			Engine::Render::pushState();
			Engine::Render::setMode(Engine::Render::MODE_GUI);

			info.print(Engine::Orientation(AVector(0, info.getH(), 1)));

			Engine::Render::popState();
			}
#endif

		Engine::Render::update();

	/**** Delta i inne framerate'y ****/
	// Przeliczanie fps'ow
#ifndef BUILD_RELEASE
		++fpsFrames;
		if(SDL_GetTicks()>fpsTimer+1000u)
			{
			char infobuff[128];

			sprintf(infobuff, "FPS: %.02f;\ttime: %.02f\nPlayer: %.0f %.0f", lfps, ltime, game.getPlayer()->getPosition().x, game.getPlayer()->getPosition().y);
			info.setStr(infobuff);
			info.update();

			lfps=fpsFrames*1000.0f/(SDL_GetTicks()-fpsTimer);
			ltime=(SDL_GetTicks()-fpsTimer)/fpsFrames;

			fpsTimer=SDL_GetTicks();
			fpsFrames=0u;
			}
#endif
		}

	Log::info("------------------------------------------------------------");
	Log::info("Zatrzymywanie...");

	game.clear();

	Engine::SpriteManager::clear();
	Engine::ImageManager::clear();
	Engine::Render::clear();
	Resource::clear();

	Log::info("Stop");
	Log::close();

	return 0;
	}

